function AclAccessProfileMenuController(eventAggregator, templateManager, requestEngine, requestFactory, Query, ZhuMenuElementsController, ZHPromise){
	var rulesRoute = 'zhuacl#/acl_zhu_acl_element_rules';
	var previousRow = {};

	function getElementsListWidget(){
		return templateManager.container.getWidget('elementsListWidget');
	}

	function getProfileRulesWidget(){
		return templateManager.container.getWidget('profileRulesWidget');
	}

	function loadProfileRules(forceUpdate){
		var profileRulesWidget = getProfileRulesWidget();
		if(forceUpdate || !profileRulesWidget.dataSource.data || !profileRulesWidget.dataSource.data.length){
			return profileRulesWidget.reload();
		}

		return ZHPromise.when(profileRulesWidget.dataSource.data);
	}

	function reloadAclRules(){
		var elementsWidget = getElementsListWidget();
		var row = elementsWidget.currentRow;
		var query = Query.build().where('ELEMENT_UNIQUE_ID').in([row.UNIQUE_ID, row.CONTAINER_ID]);
		var filterRequest = requestFactory.factory({
			serviceName: rulesRoute,
			requestType: 'filterData',
			filter: query.where()
		});
		var loadProfiles = loadProfileRules();
		var loadRules = requestEngine.doRequest(filterRequest);
		return ZHPromise.all([loadProfiles, loadRules])
			.then(function(response) {
				var defaultVisibility = templateManager.project.aclDefaultRule !== 'I';
				var rulesByGroupID = {};
				var rules = Util.getNestedProperty(response, ['1', 'dataset', 'aclZhuAccessControlRule']);
				rules.forEach(function(rule){
					var groupID = rule.GROUP_ID;
					rulesByGroupID[groupID] = rulesByGroupID[groupID] || {};
					rulesByGroupID[groupID][rule.ELEMENT_UNIQUE_ID] = {
						RULE: rule.RULE,
						RULE_ID: rule.ID
					};
				});
				getProfileRulesWidget().dataSource.data.forEach(function(data){
					var groupRule = rulesByGroupID[data.ID];
					if(groupRule){
						//menu rule
						var menuRule = groupRule[row.UNIQUE_ID] || {};
						data.VISIBILITY = menuRule.RULE ? menuRule.RULE !== 'I' : defaultVisibility;
						data.RULE = menuRule.RULE;
						data.RULE_ID = menuRule.RULE_ID;

						//container rule
						var containerRule = groupRule[row.CONTAINER_ID] || {};
						data.CONTAINER_RULE_ID = containerRule.RULE_ID;
					}
				});
			})
			.catch(function(response) {
				throw new Error(response);
			});
	}

	this.changeProfileVisibility = function(){
		var ruleRow = getElementsListWidget().currentRow;
		var profileRow = getProfileRulesWidget().currentRow;
		profileRow.changed = false;
		profileRow.VISIBILITY = !profileRow.VISIBILITY;
		var rowToSave = {
			UNIQUE_ID: ruleRow.UNIQUE_ID,
			TYPE: ruleRow.TYPE,
			RULE: profileRow.VISIBILITY ? 'A' : 'I',
			RULE_ID: profileRow.RULE_ID,
			RULE_SAVE_CONTAINER: '',
			GROUP_ID: profileRow.ID,
			LABEL: ruleRow.LABEL,
			CONTAINER_ID: ruleRow.CONTAINER_ID,
			CONTAINER_NAME: ruleRow.CONTAINER_NAME,
			CONTAINER_LABEL: ruleRow.CONTAINER_LABEL,
			CONTAINER_RULE_ID: ruleRow.CONTAINER_RULE_ID,
			NAME: ruleRow.NAME
		};
		var request = requestFactory.factory({
			requestType: 'dataSource',
			dataSource: [rowToSave],
			serviceName: rulesRoute + '/save'
		}, rulesRoute);
		requestEngine.doRequest(request).then(function(response){
			var savedRule = Util.getNestedProperty(response, ['dataset', 'aclZhuAccessControlRule']);
			if(savedRule){
				if(typeof savedRule === 'object'){
					profileRow.RULE_ID = savedRule.menuId;
					profileRow.CONTAINER_RULE_ID = savedRule.containerId;
				} else {
					profileRow.RULE_ID = savedRule;
				}
			}
		}).catch(function(err){
			profileRow.VISIBILITY = !profileRow.VISIBILITY;
		});
	};

	this.elementsListWidgetWidgetOnEnter = function(){
		eventAggregator.subscribe('menuBuilded', ZhuMenuElementsController.loadPlanningTree);
	};

	this.beforeMoveRowMenu = function(row){
		previousRow = row;
	};

	this.afterMoveRowMenu = function(widget, row){
		if(row.buildedMenu || row.TYPE === 'G' || row._childCount){
			widget.currentRow = previousRow;
		} else {
			reloadAclRules();
		}
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register('AclAccessProfileMenuController', AclAccessProfileMenuController);
});
